"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var hooks = {
  read: [],
  write: [],
  update: [],
  request: []
};

/**
 * An empty plugin object. Every plugin group
 * will boil down to one of these.
 * @type {Object}
 */
var base = {

  /**
   * Lifecycle hooks.
   * @type {Object}
   */
  hooks: {
    before: hooks,
    after: hooks,
    catch: hooks
  },

  /**
   * A list of storage drivers.
   * @type {Array}
   */
  storage: [],

  /**
   * API extensions.
   * @type {Object}
   */
  extend: {
    root: {},
    context: {}
  },

  /**
   * Network drivers.
   * @type {Object}
   */
  network: {
    clients: [],
    servers: []
  }
};

var merge = {

  /**
   * Merges a hook object with another.
   * @param  {Object} hook - The base hook to extend.
   * @param  {Object} hook.read - The base hook to extend.
   * @param  {Object} hook.write - The base hook to extend.
   * @param  {Object} hook.update - The base hook to extend.
   * @param  {Object} hook.request - The base hook to extend.
   * @param  {Object} target={} - The hooks to add.
   * @return {Object} - The merged hook object.
   */
  hook: function hook(_ref) {
    var read = _ref.read,
        write = _ref.write,
        update = _ref.update,
        request = _ref.request;
    var target = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return {
      read: read.concat(target.read || []),
      write: write.concat(target.write || []),
      update: update.concat(target.update || []),
      request: request.concat(target.request || [])
    };
  },

  /**
   * Merges hook objects together.
   * @param  {Object} hooks - Database lifecycle hooks.
   * @param  {Object} hooks.before - Hooks to run before an event.
   * @param  {Object} hooks.after - Hooks to run after an event.
   * @param  {Object} hooks.catch - Hooks to run after an error.
   * @param  {Object} ext={} - Hooks to merge with the base.
   * @return {Object} - The merged hooks object.
   */
  hooks: function hooks(_hooks) {
    var ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return {
      before: merge.hook(_hooks.before, ext.before),
      after: merge.hook(_hooks.after, ext.after),
      catch: merge.hook(_hooks.catch, ext.catch)
    };
  },

  /**
   * Merges two method objects.
   * @param  {Object} base - The base object to extend.
   * @param  {Object} ext={} - An object with method extensions.
   * @param  {Object} ext.root - Methods to add to a root context.
   * @param  {Object} ext.context - Methods for node contexts.
   * @return {Object} - The combined extensions object.
   */
  extensions: function extensions(_ref2) {
    var root = _ref2.root,
        context = _ref2.context;
    var ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return {
      root: (0, _assign2.default)(root, ext.root),
      context: (0, _assign2.default)(context, ext.context)
    };
  },

  /**
   * Merges two network configurations together.
   * @param  {Object} base - The base network object to extend.
   * @param  {Array} base.clients - A list of network clients.
   * @param  {Array} base.servers - A list of network servers.
   * @param  {Object} ext={} - Extensions to the base object.
   * @return {Object} - The merged network object.
   */
  network: function network(_ref3) {
    var clients = _ref3.clients,
        servers = _ref3.servers;
    var ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return {
      clients: clients.concat(ext.clients || []),
      servers: servers.concat(ext.servers || [])
    };
  },

  /**
   * Merges two storage arrays together.
   * @param  {Array} base - A list of storage interfaces.
   * @param  {Array} ext=[] - A list of storage interfaces.
   * @return {Array} - The merged list.
   */
  storage: function storage(base) {
    var ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    return base.concat(ext);
  },

  /**
   * Merges two plugin objects together.
   * @param  {Object} base - The base plugin object to extend.
   * @param  {Object} base.hooks - Database lifecycle hooks.
   * @param  {Array} base.storage - A list of storage drivers.
   * @param  {Object} base.network - A list of network drivers.
   * @param  {Object} base.extend - Method extensions.
   * @param  {Object} ext - An object to merge with the base.
   * @return {Object} - The complete, merged plugin object.
   */
  plugins: function plugins(base, ext) {
    return {
      hooks: merge.hooks(base.hooks, ext.hooks),
      storage: merge.storage(base.storage, ext.storage),
      extend: merge.extensions(base.extend, ext.extend),
      network: merge.network(base.network, ext.network)
    };
  }
};

/**
 * Merges plugin objects into a single object.
 * @param  {Object[]} plugins - A list of plugin objects.
 * @return {Object} - The merged plugin object.
 */

exports.default = function (plugins) {
  return plugins.reduce(merge.plugins, base);
};