'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.after = exports.before = exports.format = exports.defaults = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _triggerHooks = require('../trigger-hooks');

var _triggerHooks2 = _interopRequireDefault(_triggerHooks);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Sets default options for methods like `.read` & `.write`.
 * @param  {Object} config - The database configuration object.
 * @param  {Object} [options] - Options passed to read/write.
 * @return {Object} - An options object with defaults set via the config.
 */
var defaults = exports.defaults = function defaults(config) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return (0, _extends3.default)({}, options, {
    storage: options.storage || config.storage,
    clients: options.clients || config.network.clients
  });
};

/**
 * Adds default options to the last argument value (typically options).
 * @param  {Object} config - The database configuration object.
 * @param  {Array} args - A list of arguments.
 * @return {Array} - Newly arguments, with default options added.
 */
var normalizeArgs = function normalizeArgs(config) {
  var args = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  var last = args[args.length - 1];

  return args.slice(0, args.length - 1).concat([defaults(config, last)]);
};

/**
 * Handles common return values from pipeline hooks.
 * @param  {Function} map - Turns non-array return values into an array.
 * @return {Function} - Formats output from one hook for the next.
 */
var format = exports.format = function format(map) {
  return (

    /**
     * Takes output from one hook and formats it for the next.
     * @param  {Mixed} next - Anything a hook can return.
     * @param  {Array} last - The input to the last hook.
     * @return {Array} - The arguments for the next hook.
     */
    function (next, last) {

      /** If an array isn't returned, turn it into one. */
      if (!(next instanceof Array)) {
        next = map(next);
      }

      /** Uses the last input to patch any missing arguments. */
      return last.map(function (prev, index) {
        if (next[index] === undefined) {
          return prev;
        }

        return next[index];
      });
    }
  );
};

/**
 * Ignore a pipeline return value, skipping to the next.
 * @return {Array} - An empty list of arguments.
 */
var skip = function skip() {
  return [];
};

/**
 * Creates an event pipeline.
 * @param  {String} tense - The verb tense.
 * @param  {String} action - The action being dispatched.
 * @param  {Function} [types] - Handles special hook return values.
 * @return {Function} - Triggers a pipeline.
 */
var createPipeline = function createPipeline(tense, action) {
  var types = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : skip;
  return (

    /**
     * Triggers a pipeline.
     * @param  {Object} config - Database configuration object.
     * @param  {Array} args - The initial parameters.
     * @return {Promise} - Resolves to the pipeline output.
     */
    function (config, args) {
      return (0, _triggerHooks2.default)({
        hooks: config.hooks[tense][action],
        args: normalizeArgs(config, args),
        transform: format(types)
      });
    }
  );
};

var before = exports.before = {

  read: createPipeline('before', 'read', function (value) {

    // Override the key.
    if (typeof value === 'string') {
      return [value];
    }

    // Override the options.
    if (value instanceof Object) {
      return [undefined, value];
    }

    return null;
  }),

  write: createPipeline('before', 'write'),
  request: createPipeline('before', 'request'),
  update: createPipeline('before', 'update')

};

var after = exports.after = {

  read: createPipeline('after', 'read'),
  write: createPipeline('after', 'write'),
  request: createPipeline('after', 'request'),
  update: createPipeline('after', 'update')

};